# Copyright (c) 2009-2012, Geoffrey Biggs
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Geoffrey Biggs nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# File: props.py
# Author: Geoffrey Biggs
# Part of pykg-config.

"""Defines the empty property dictionaries.

These are defined here to keep them together for ease of maintenance
when new properties are added.

"""

__version__ = "$Revision: $"
# $Source$

from pykg_config.version import Version

empty_raw_props = {'name': '',
                   'description': '',
                   'url': '',
                   'version': '',
                   'requires': '',
                   'requires.private': '',
                   'conflicts': '',
                   'cflags': '',
                   'libs': '',
                   'libs.private': ''}
empty_processed_props = {'name': '',
                         'description': 'No description',
                         'url': '',
                         'version': Version(),
                         'requires': [],
                         'requires.private': [],
                         'conflicts': [],
                         'include_dirs': [],
                         'other_cflags': [],
                         'libs': [],
                         'libpaths': [],
                         'otherlibs': [],
                         'private.libs': [],
                         'private.libpaths': [],
                         'private.otherlibs': []}


# vim: tw=79

